@@ -2,15 +2,15 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 7)
+__version__ = (0, 0, 8)
 __all__ = ["ApplicationWithMethods", "make_application"]
 
 from asyncio import gather
-from collections.abc import Callable, Iterable, Iterator, Sequence
+from collections.abc import AsyncIterator, Callable, Iterable, Iterator, Sequence
 from functools import cached_property, partial
 from itertools import chain
 from sys import _getframe, maxsize
-from typing import overload, SupportsIndex, TypeVar
+from typing import overload, Literal, SupportsIndex, TypeVar
 
 from blacksheep import Application, Request, Router, WebSocket
 from blacksheep.contents import StreamedContent
@@ -164,7 +164,7 @@ class ApplicationWithMethods(Application):
 def make_application(
     base_url: str = "http://localhost", 
     custom_proxy: bool | Callable = True, 
-    duplex_ws: None | bool = False, 
+    ws_mode: None | Literal["", "r", "w", "rw"] = None, 
     debug: bool = False, 
 ) -> ApplicationWithMethods:
     """创建一个 blacksheep 应用，用于反向代理 emby，以修改一些数据响应
@@ -176,17 +176,21 @@ def make_application(
         - 如果为 True，则提供一个默认的方法，会被绑定 "/" 和 "/<path:path>" 路由路径到任何请求方法
         - 如果为 Callable，会被绑定 "/" 和 "/<path:path>" 路由路径到任何请求方法
 
-    :param duplex_ws: 是否为双向 websocket
+    :param ws_mode: websocket 的读写模式
 
-        - 如果为 None，则不代理 websocket
-        - 如果为 False，则代理为单向 websocket，只进行接收
-        - 如果为 True，则代理为双向 websocket，会进行接收和发送
+        - 如果为 None 或 ""，则不代理 websocket
+        - 如果为 "r"，则代理为单向 websocket，只进行接收（读）
+        - 如果为 "r"，则代理为单向 websocket，只进行发送（写）
+        - 如果为 "rw"，则代理为双向 websocket，会进行接收（读）和发送（写）
 
     :param debug: 启用调试，会输出 DEBUG 级别日志，也会产生更详细的报错信息
 
     :return: 一个 blacksheep 应用，你可以 2 次扩展，并用 uvicorn 运行
     """
+    if not base_url.startswith(("http://", "https://")):
+        raise ValueError(f"not a valid http/https base_url: {base_url!r}")
     base_url = base_url.rstrip("/")
+
     router = Router()
     app = ApplicationWithMethods(router=router, show_error_details=debug)
 
@@ -194,7 +198,11 @@ def make_application(
         getattr(app, "logger").level = 10
 
     @app.add_method
-    async def redirect_request(request: Request, data=None) -> HTTPResponse:
+    async def redirect_request(
+        request: Request, 
+        data: None | bytes | AsyncIterator[bytes] = None, 
+        timeout: None | int | float = None, 
+    ) -> HTTPResponse:
         proxy_base_url = f"{request.scheme}://{request.host}"
         request_headers = [
             (k, base_url + v[len(proxy_base_url):] if k in ("destination", "origin", "referer") and v.startswith(proxy_base_url) else v)
@@ -208,7 +216,7 @@ def make_application(
                 url=base_url+str(request.url), 
                 data=request.stream() if data is None else data, # type: ignore
                 headers=request_headers, 
-                timeout=None, 
+                timeout=timeout, 
             ), 
             follow_redirects=False, 
             stream=True,
@@ -254,23 +262,17 @@ def make_application(
             content=StreamedContent(bytes(content_type, "latin-1"), reader), 
         ), data)
 
-    @app.on_middlewares_configuration
-    def configure_forwarded_headers(app: Application):
-        app.middlewares.insert(0, ForwardedHeadersMiddleware(accept_only_proxied_requests=False))
-
-    @app.lifespan
-    async def register_http_client(app: Application):
-        async with AsyncClient() as session:
-            app.services.register(AsyncClient, instance=session)
-            yield
-
-    @router.route("/_redirect", methods=[""])
+    @app.add_method
     async def redirect(
         request: Request, 
-        session: AsyncClient, 
-        url: str, 
-        timeout: None | float = None, 
-    ):
+        url: None | str = None, 
+        timeout: None | int | float = None, 
+    ) -> Response:
+        if url is None:
+            http_resp = await redirect_request(request, timeout=timeout)
+            response, _ = await make_response(request, http_resp)
+            return response
+        session = app.services.resolve(AsyncClient)
         resp = await session.send(
             request=session.build_request(
                 method=request.method, 
@@ -306,7 +308,51 @@ def make_application(
             content=StreamedContent(bytes(content_type, "latin-1"), reader), 
         )
 
-    if duplex_ws is not None:
+    @app.add_method
+    async def redirect_websocket(
+        request: Request, 
+        ws_to: WebSocket, 
+        ws_mode: Literal["r", "w", "rw"] = "rw", 
+    ):
+        await ws_to.accept()
+        proxy_base_url = f"{request.scheme.replace('ws', 'http')}://{request.host}"
+        async with websocket_connect(
+            "ws" + base_url.removeprefix("http") + str(request.url), 
+            additional_headers=[
+                (k, base_url + v[len(proxy_base_url):] if k in ("destination", "origin", "referer") and v.startswith(proxy_base_url) else v)
+                for k, v in ((str(k, "latin-1"), str(v, "latin-1")) for k, v in request.headers)
+                if k.lower() not in ("host", "sec-websocket-key")
+            ], 
+        ) as ws_from:
+            async def redirect(recv, send):
+                try:
+                    while True:
+                        await send(await recv())
+                except Exception as e:
+                    exc_cls = (WebSocketDisconnectError, ConnectionClosed)
+                    if not (isinstance(e, exc_cls) or isinstance(e.__cause__, exc_cls)):
+                        raise
+            if ws_mode == "r":
+                await redirect(ws_from.recv, ws_to.send_bytes)
+            elif ws_mode == "w":
+                await redirect(ws_to.receive_bytes, ws_from.send)
+            else:
+                await gather(
+                    redirect(ws_from.recv, ws_to.send_bytes), 
+                    redirect(ws_to.receive_bytes, ws_from.send), 
+                )
+
+    @app.on_middlewares_configuration
+    def configure_forwarded_headers(app: Application):
+        app.middlewares.insert(0, ForwardedHeadersMiddleware(accept_only_proxied_requests=False))
+
+    @app.lifespan
+    async def register_http_client(app: Application):
+        async with AsyncClient() as session:
+            app.services.register(AsyncClient, instance=session)
+            yield
+
+    if ws_mode:
         @router.ws("/")
         @router.ws("/<path:path>")
         async def proxy(
@@ -314,31 +360,7 @@ def make_application(
             ws_to: WebSocket, 
             path: str = "", 
         ):
-            await ws_to.accept()
-            proxy_base_url = f"{request.scheme.replace('ws', 'http')}://{request.host}"
-            async with websocket_connect(
-                "ws" + base_url.removeprefix("http") + str(request.url), 
-                additional_headers=[
-                    (k, base_url + v[len(proxy_base_url):] if k in ("destination", "origin", "referer") and v.startswith(proxy_base_url) else v)
-                    for k, v in ((str(k, "latin-1"), str(v, "latin-1")) for k, v in request.headers)
-                    if k.lower() not in ("host", "sec-websocket-key")
-                ], 
-            ) as ws_from:
-                async def redirect(recv, send):
-                    try:
-                        while True:
-                            await send(await recv())
-                    except Exception as e:
-                        exc_cls = (WebSocketDisconnectError, ConnectionClosed)
-                        if not (isinstance(e, exc_cls) or isinstance(e.__cause__, exc_cls)):
-                            raise
-                if duplex_ws:
-                    await gather(
-                        redirect(ws_from.recv, ws_to.send_bytes), 
-                        redirect(ws_to.receive_bytes, ws_from.send), 
-                    )
-                else:
-                    await redirect(ws_from.recv, ws_to.send_bytes)
+            await redirect_websocket(request, ws_to, ws_mode)
 
     if callable(custom_proxy):
         router.route("/", methods=[""])(custom_proxy)
@@ -346,15 +368,10 @@ def make_application(
     elif custom_proxy:
         @router.route("/", methods=[""])
         @router.route("/<path:path>", methods=[""])
-        async def proxy(
-            request: Request, 
-            session: AsyncClient, 
-            path: str = "", 
-        ):
-            http_resp = await redirect_request(request)
-            response, _ = await make_response(request, http_resp)
-            return response
+        async def proxy(request: Request, path: str = ""):
+            return await redirect(request)
 
     return app
 
 # TODO: 再实现一个版本，用 fastapi，因为这个框架更新较频繁，可以在 python 3.13 上稳定运行，也可以在多种设备上编译成功
+# TODO: 再实现一个版本，用 emmett
